@@ -1,15 +1,17 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2020 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author:
-#  Alberto Solino (@agsolino)
-#
 # Description:
-#       Example for using the DPAPI/Vault structures to unlock Windows Secrets.
+#   Example for using the DPAPI/Vault structures to unlock Windows Secrets.
+#
+# Author:
+#   Alberto Solino (@agsolino)
 #
 # Examples:
 #
@@ -20,14 +22,16 @@
 #   In the case of vaults, you might need to also provide the user's sid (and the user password will be asked).
 #   For system secrets, instead of a password you will need to specify the system and security hives.
 #
-# References: All of the work done by these guys. I just adapted their work to my needs.
-#       https://www.passcape.com/index.php?section=docsys&cmd=details&id=28
-#       https://github.com/jordanbtucker/dpapick
-#       https://github.com/gentilkiwi/mimikatz/wiki/howto-~-credential-manager-saved-credentials (and everything else Ben did )
-#       http://blog.digital-forensics.it/2016/01/windows-revaulting.html
-#       https://www.passcape.com/windows_password_recovery_vault_explorer
-#       https://www.passcape.com/windows_password_recovery_dpapi_master_key
+# References:
+#   All of the work done by these guys. I just adapted their work to my needs.
+#   - https://www.passcape.com/index.php?section=docsys&cmd=details&id=28
+#   - https://github.com/jordanbtucker/dpapick
+#   - https://github.com/gentilkiwi/mimikatz/wiki/howto-~-credential-manager-saved-credentials (and everything else Ben did )
+#   - http://blog.digital-forensics.it/2016/01/windows-revaulting.html
+#   - https://www.passcape.com/windows_password_recovery_vault_explorer
+#   - https://www.passcape.com/windows_password_recovery_dpapi_master_key
 #
+
 from __future__ import division
 from __future__ import print_function
 
@@ -314,7 +318,7 @@ class DPAPI:
                         break
                 masterkey=b''.join(resp['ppDataOut'][beginning:])
                 print('Decrypted key using rpc call')
-                print('Decrypted key: 0x%s' % hexlify(masterkey[beginning:]).decode())
+                print('Decrypted key: 0x%s' % hexlify(masterkey).decode())
                 return
 
             else:
@@ -486,7 +490,7 @@ class DPAPI:
                     entropy = fp2.read()
                     fp2.close()
                 elif self.options.entropy is not None:
-                    entropy = b(self.options.entropy) + b'\x00'
+                    entropy = b(self.options.entropy)
                 else:
                     entropy = None
 
@@ -507,7 +511,7 @@ if __name__ == '__main__':
     logger.init()
     print(version.BANNER)
 
-    parser = argparse.ArgumentParser(add_help=True, description="Nose")
+    parser = argparse.ArgumentParser(add_help=True, description="Example for using the DPAPI/Vault structures to unlock Windows Secrets.")
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     subparsers = parser.add_subparsers(help='actions', dest='action')
 
