@@ -1,23 +1,26 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
-#   [MS-COMEV]: Component Object Model Plus (COM+) Event System Protocol. 
-#               This was used as a way to test the DCOM runtime. Further 
+#   [MS-COMEV]: Component Object Model Plus (COM+) Event System Protocol.
+#               This was used as a way to test the DCOM runtime. Further
 #               testing is needed to verify it is working as expected
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
+#   at https://github.com/fortra/impacket/tree/master/tests/SMB_RPC
 #
-#   Since DCOM is like an OO RPC, instead of helper functions you will see the 
-#   classes described in the standards developed. 
-#   There are test cases for them too. 
+#   Since DCOM is like an OO RPC, instead of helper functions you will see the
+#   classes described in the standards developed.
+#   There are test cases for them too.
+#
+# Author:
+#   Alberto Solino (@agsolino)
 #
 from __future__ import division
 from __future__ import print_function
@@ -1728,7 +1731,7 @@ class IEnumEventObject(IDispatch):
     def Clone(self):
         request = IEnumEventObject_Clone()
         resp = self.request(request, iid = self._iid, uuid = self.get_iPid())
-        return IEnumEventObject(INTERFACE(self.get_cinstance(), ''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
+        return IEnumEventObject(INTERFACE(self.get_cinstance(), b''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
 
     def Next(self, cReqElem):
         request = IEnumEventObject_Next()
@@ -1736,7 +1739,7 @@ class IEnumEventObject(IDispatch):
         resp = self.request(request, iid = self._iid, uuid = self.get_iPid())
         interfaces = list()
         for interface in resp['ppInterface']:
-            interfaces.append(IEventClass2(INTERFACE(self.get_cinstance(), ''.join(interface['abData']), self.get_ipidRemUnknown(), target = self.get_target())))
+            interfaces.append(IEventClass2(INTERFACE(self.get_cinstance(), b''.join(interface['abData']), self.get_ipidRemUnknown(), target = self.get_target())))
         return interfaces
 
     def Reset(self):
@@ -1758,7 +1761,7 @@ class IEventObjectCollection(IDispatch):
     def get__NewEnum(self):
         request = IEventObjectCollection_get__NewEnum()
         resp = self.request(request, iid = self._iid , uuid = self.get_iPid())
-        return IEnumEventObject(INTERFACE(self.get_cinstance(), ''.join(resp['ppEnum']['abData']), self.get_ipidRemUnknown(), target = self._get_target()))
+        return IEnumEventObject(INTERFACE(self.get_cinstance(), b''.join(resp['ppEnum']['abData']), self.get_ipidRemUnknown(), target = self._get_target()))
 
     def get_Item(self, objectID):
         request = IEventObjectCollection_get_Item()
@@ -1769,7 +1772,7 @@ class IEventObjectCollection(IDispatch):
     def get_NewEnum(self):
         request = IEventObjectCollection_get_NewEnum()
         resp = self.request(request, iid = self._iid , uuid = self.get_iPid())
-        return IEnumEventObject(INTERFACE(self.get_cinstance(), ''.join(resp['ppEnum']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
+        return IEnumEventObject(INTERFACE(self.get_cinstance(), b''.join(resp['ppEnum']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
 
     def get_Count(self):
         request = IEventObjectCollection_get_Count()
@@ -1799,7 +1802,7 @@ class IEventSystem(IDispatch):
         request['progID']['asData']=progID
         request['queryCriteria']['asData']=queryCriteria
         resp = self.request(request, iid = self._iid, uuid = self.get_iPid())
-        iInterface = IDispatch(INTERFACE(self.get_cinstance(), ''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
+        iInterface = IDispatch(INTERFACE(self.get_cinstance(), b''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
         return IEventObjectCollection(iInterface.RemQueryInterface(1, (IID_IEventObjectCollection,)))
 
     def Store(self, progID, pInterface):
@@ -1826,7 +1829,7 @@ class IEventSystem(IDispatch):
         request['progID']['asData']=progID
         request['queryCriteria']['asData']=queryCriteria
         resp = self.request(request, uuid = self.get_iPid())
-        iInterface = IDispatch(INTERFACE(self.get_cinstance(), ''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
+        iInterface = IDispatch(INTERFACE(self.get_cinstance(), b''.join(resp['ppInterface']['abData']), self.get_ipidRemUnknown(), target = self.get_target()))
         return IEventObjectCollection(iInterface.RemQueryInterface(1, (IID_IEventObjectCollection,)))
 
     def RemoveS(self,progID, queryCriteria):
