@@ -1,24 +1,27 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#         Itamar Mizrahi (@MrAnde7son)
-#
 # Description:
 #   [MS-EVEN] Interface implementation
 #
 #   Best way to learn how to use these calls is to grab the protocol standard
 #   so you understand what the call does, and then read the test case located
-#   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
+#   at https://github.com/fortra/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
 #   They are located at the end of this file.
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too.
 #
+# Author:
+#   Alberto Solino (@agsolino)
+#   Itamar Mizrahi (@MrAnde7son)
+#
 from __future__ import division
 from __future__ import print_function
 from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDR, NDRPOINTERNULL, NDRUniConformantArray
@@ -366,7 +369,7 @@ def hElfrRegisterEventSourceW(dce, moduleName = NULL, regModuleName = NULL):
     request['MinorVersion'] = 1
     return dce.request(request)
 
-def hElfrReadELW(dce, logHandle = '', readFlags = EVENTLOG_SEQUENTIAL_READ|EVENTLOG_FORWARDS_READ,
+def hElfrReadELW(dce, logHandle = '', readFlags = EVENTLOG_SEEK_READ|EVENTLOG_FORWARDS_READ,
                  recordOffset = 0, numberOfBytesToRead = MAX_BATCH_BUFF):
     request = ElfrReadELW()
     request['LogHandle'] = logHandle
