@@ -1,14 +1,17 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   Transport implementations for the DCE/RPC protocol.
 #
+# Author:
+#   Alberto Solino (@agsolino)
+#
 from __future__ import division
 from __future__ import print_function
 
@@ -17,16 +20,22 @@ import os
 import re
 import socket
 
+try:
+    from urllib.parse import urlparse, urlunparse
+except ImportError:
+    from urlparse import urlparse, urlunparse
+
 from impacket import ntlm
 from impacket.dcerpc.v5.rpcrt import DCERPCException, DCERPC_v5, DCERPC_v4
+from impacket.dcerpc.v5.rpch import RPCProxyClient, RPCProxyClientException, RPC_OVER_HTTP_v1, RPC_OVER_HTTP_v2
 from impacket.smbconnection import SMBConnection
 
 
 class DCERPCStringBinding:
-    parser = re.compile(r'(?:([a-fA-F0-9-]{8}(?:-[a-fA-F0-9-]{4}){3}-[a-fA-F0-9-]{12})@)?' # UUID (opt.)
-                        +'([_a-zA-Z0-9]*):' # Protocol Sequence
-                        +'([^\[]*)' # Network Address (opt.)
-                        +'(?:\[([^\]]*)\])?') # Endpoint and options (opt.)
+    parser = re.compile(r"(?:([a-fA-F0-9-]{8}(?:-[a-fA-F0-9-]{4}){3}-[a-fA-F0-9-]{12})@)?" +  # UUID (opt.)
+                        r"([_a-zA-Z0-9]*):" +  # Protocol Sequence
+                        r"([^\[]*)" +  # Network Address (opt.)
+                        r"(?:\[([^]]*)])?")  # Endpoint and options (opt.)
 
     def __init__(self, stringbinding):
         match = DCERPCStringBinding.parser.match(stringbinding)
@@ -36,16 +45,21 @@ class DCERPCStringBinding:
         options = match.group(4)
         if options:
             options = options.split(',')
+            
             self.__endpoint = options[0]
             try:
                 self.__endpoint.index('endpoint=')
                 self.__endpoint = self.__endpoint[len('endpoint='):]
             except:
                 pass
-            self.__options = options[1:]
+
+            self.__options = {}
+            for option in options[1:]:
+                vv = option.split('=', 1)
+                self.__options[vv[0]] = vv[1] if len(vv) > 1 else ''
         else:
             self.__endpoint = ''
-            self.__options = []
+            self.__options = {}
 
     def get_uuid(self):
         return self.__uuid
@@ -56,16 +70,29 @@ class DCERPCStringBinding:
     def get_network_address(self):
         return self.__na
 
+    def set_network_address(self, addr):
+        self.__na = addr
+
     def get_endpoint(self):
         return self.__endpoint
 
     def get_options(self):
         return self.__options
 
+    def get_option(self, option_name):
+        return self.__options[option_name]
+
+    def is_option_set(self, option_name):
+        return option_name in self.__options
+
+    def unset_option(self, option_name):
+        del self.__options[option_name]
+
     def __str__(self):
         return DCERPCStringBindingCompose(self.__uuid, self.__ps, self.__na, self.__endpoint, self.__options)
 
-def DCERPCStringBindingCompose(uuid=None, protocol_sequence='', network_address='', endpoint='', options=[]):
+
+def DCERPCStringBindingCompose(uuid=None, protocol_sequence='', network_address='', endpoint='', options={}):
     s = ''
     if uuid:
         s += uuid + '@'
@@ -75,11 +102,12 @@ def DCERPCStringBindingCompose(uuid=None, protocol_sequence='', network_address=
     if endpoint or options:
         s += '[' + endpoint
         if options:
-            s += ',' + ','.join(options)
+            s += ',' + ','.join([key if str(val) == '' else "=".join([key, str(val)]) for key, val in options.items()])
         s += ']'
 
     return s
 
+
 def DCERPCTransportFactory(stringbinding):
     sb = DCERPCStringBinding(stringbinding)
 
@@ -88,34 +116,36 @@ def DCERPCTransportFactory(stringbinding):
     if 'ncadg_ip_udp' == ps:
         port = sb.get_endpoint()
         if port:
-            return UDPTransport(na, int(port))
+            rpctransport = UDPTransport(na, int(port))
         else:
-            return UDPTransport(na)
+            rpctransport = UDPTransport(na)
     elif 'ncacn_ip_tcp' == ps:
         port = sb.get_endpoint()
         if port:
-            return TCPTransport(na, int(port))
+            rpctransport = TCPTransport(na, int(port))
         else:
-            return TCPTransport(na)
+            rpctransport = TCPTransport(na)
     elif 'ncacn_http' == ps:
         port = sb.get_endpoint()
         if port:
-            return HTTPTransport(na, int(port))
+            rpctransport = HTTPTransport(na, int(port))
         else:
-            return HTTPTransport(na)
+            rpctransport = HTTPTransport(na)
     elif 'ncacn_np' == ps:
         named_pipe = sb.get_endpoint()
         if named_pipe:
             named_pipe = named_pipe[len(r'\pipe'):]
-            return SMBTransport(na, filename = named_pipe)
+            rpctransport = SMBTransport(na, filename = named_pipe)
         else:
-            return SMBTransport(na)
+            rpctransport = SMBTransport(na)
     elif 'ncalocal' == ps:
         named_pipe = sb.get_endpoint()
-        return LOCALTransport(filename = named_pipe)
+        rpctransport = LOCALTransport(filename = named_pipe)
     else:
         raise DCERPCException("Unknown protocol sequence.")
 
+    rpctransport.set_stringbinding(sb)
+    return rpctransport
 
 class DCERPCTransport:
 
@@ -125,6 +155,7 @@ class DCERPCTransport:
         self.__remoteName = remoteName
         self.__remoteHost = remoteName
         self.__dstport = dstport
+        self._stringbinding = None
         self._max_send_frag = None
         self._max_recv_frag = None
         self._domain = ''
@@ -140,6 +171,11 @@ class DCERPCTransport:
         self._TGS      = None
         self._kdcHost  = None
         self.set_credentials('','')
+        # Strict host validation - off by default and currently only for
+        # SMBTransport
+        self._strict_hostname_validation = False
+        self._validation_allow_absent = True
+        self._accepted_hostname = ''
 
     def connect(self):
         raise RuntimeError('virtual function')
@@ -177,6 +213,12 @@ class DCERPCTransport:
         """This method only makes sense before connection for most protocols."""
         self.__dstport = dport
 
+    def get_stringbinding(self):
+        return self._stringbinding
+
+    def set_stringbinding(self, stringbinding):
+        self._stringbinding = stringbinding
+
     def get_addr(self):
         return self.getRemoteHost(), self.get_dport()
     def set_addr(self, addr):
@@ -203,6 +245,11 @@ class DCERPCTransport:
         else:
             self._max_send_frag = send_fragment_size
 
+    def set_hostname_validation(self, validate, accept_empty, hostname):
+        self._strict_hostname_validation = validate
+        self._validation_allow_absent = accept_empty
+        self._accepted_hostname = hostname
+
     def set_default_max_fragment_size(self):
         # default is 0: don't fragment.
         # subclasses may override this method
@@ -216,7 +263,7 @@ class DCERPCTransport:
             self._lmhash,
             self._nthash,
             self._aesKey,
-            self._TGT, 
+            self._TGT,
             self._TGS)
 
     def set_credentials(self, username, password, domain='', lmhash='', nthash='', aesKey='', TGT=None, TGS=None):
@@ -340,16 +387,84 @@ class TCPTransport(DCERPCTransport):
     def get_socket(self):
         return self.__socket
 
-class HTTPTransport(TCPTransport):
+class HTTPTransport(TCPTransport, RPCProxyClient):
     """Implementation of ncacn_http protocol sequence"""
 
-    def connect(self):
-        TCPTransport.connect(self)
+    def __init__(self, remoteName=None, dstport=593):
+        self._useRpcProxy = False
+        self._rpcProxyUrl = None
+        self._transport   = TCPTransport
+        self._version     = RPC_OVER_HTTP_v2
 
-        self.get_socket().send('RPC_CONNECT ' + self.getRemoteHost() + ':593 HTTP/1.0\r\n\r\n')
-        data = self.get_socket().recv(8192)
-        if data[10:13] != '200':
-            raise DCERPCException("Service not supported.")
+        DCERPCTransport.__init__(self, remoteName, dstport)
+        RPCProxyClient.__init__(self, remoteName, dstport)
+        self.set_connect_timeout(30)
+
+    def set_credentials(self, username, password, domain='', lmhash='', nthash='', aesKey='', TGT=None, TGS=None):
+        return self._transport.set_credentials(self, username, password,
+            domain, lmhash, nthash, aesKey, TGT, TGS)
+
+    def rpc_proxy_init(self):
+        self._useRpcProxy = True
+        self._transport   = RPCProxyClient
+
+    def set_rpc_proxy_url(self, url):
+        self.rpc_proxy_init()
+        self._rpcProxyUrl = urlparse(url)
+
+    def get_rpc_proxy_url(self):
+        return urlunparse(self._rpcProxyUrl)
+
+    def set_stringbinding(self, set_stringbinding):
+        DCERPCTransport.set_stringbinding(self, set_stringbinding)
+
+        if self._stringbinding.is_option_set("RpcProxy"):
+            self.rpc_proxy_init()
+
+            rpcproxy = self._stringbinding.get_option("RpcProxy").split(":")
+
+            if rpcproxy[1] == '443':
+                self.set_rpc_proxy_url('https://%s/rpc/rpcproxy.dll' % rpcproxy[0])
+            elif rpcproxy[1] == '80':
+                self.set_rpc_proxy_url('http://%s/rpc/rpcproxy.dll' % rpcproxy[0])
+            else:
+                # 2.1.2.1
+                # RPC over HTTP always uses port 80 for HTTP traffic and port 443 for HTTPS traffic.
+                # But you can use set_rpc_proxy_url method to set any URL / query you want.
+                raise DCERPCException("RPC Proxy port must be 80 or 443")
+
+    def connect(self):
+        if self._useRpcProxy == False:
+            # Connecting directly to the ncacn_http port
+            #
+            # Here we using RPC over HTTPv1 instead complex RPC over HTTP v2 syntax
+            # RPC over HTTP v2 here can be implemented in the future
+            self._version = RPC_OVER_HTTP_v1
+
+            TCPTransport.connect(self)
+
+            # Reading legacy server response
+            data = self.get_socket().recv(8192)
+
+            if data != b'ncacn_http/1.0':
+                raise DCERPCException("%s:%s service is not ncacn_http" % (self.__remoteName, self.__dstport))
+        else:
+            RPCProxyClient.connect(self)
+
+    def send(self, data, forceWriteAndx=0, forceRecv=0):
+        return self._transport.send(self, data, forceWriteAndx, forceRecv)
+
+    def recv(self, forceRecv=0, count=0):
+        return self._transport.recv(self, forceRecv, count)
+
+    def get_socket(self):
+        if self._useRpcProxy == False:
+            return TCPTransport.get_socket(self)
+        else:
+            raise DCERPCException("This method is not supported for RPC Proxy connections")
+
+    def disconnect(self):
+        return self._transport.disconnect(self)
 
 class SMBTransport(DCERPCTransport):
     """Implementation of ncacn_np protocol sequence"""
@@ -377,6 +492,7 @@ class SMBTransport(DCERPCTransport):
 
         self.__prefDialect = None
         self.__smb_connection = smb_connection
+        self.set_connect_timeout(30)
 
     def preferred_dialect(self, dialect):
         self.__prefDialect = dialect
@@ -384,7 +500,9 @@ class SMBTransport(DCERPCTransport):
     def setup_smb_connection(self):
         if not self.__smb_connection:
             self.__smb_connection = SMBConnection(self.getRemoteName(), self.getRemoteHost(), sess_port=self.get_dport(),
-                                                  preferredDialect=self.__prefDialect)
+                                                  preferredDialect=self.__prefDialect, timeout=self.get_connect_timeout())
+            if self._strict_hostname_validation:
+                self.__smb_connection.setHostnameValidation(self._strict_hostname_validation, self._validation_allow_absent, self._accepted_hostname)
 
     def connect(self):
         # Check if we have a smb connection already setup
