@@ -1,18 +1,18 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# Copyright (C) 2023 Fortra. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# A Socks Proxy for the IMAPS Protocol
+# Description:
+#   Socks Proxy for the IMAPS Protocol
+#
+#   A simple SOCKS server that proxies a connection to relayed IMAPS connections
 #
 # Author:
-#  Dirk-jan Mollema (@_dirkjan) / Fox-IT (https://www.fox-it.com)
-#
-# Description:
-#  A simple SOCKS server that proxies a connection to relayed IMAPS connections
-#
-# ToDo:
+#   Dirk-jan Mollema (@_dirkjan) / Fox-IT (https://www.fox-it.com)
 #
 from impacket import LOG
 from impacket.examples.ntlmrelayx.servers.socksplugins.imap import IMAPSocksRelay
@@ -43,7 +43,7 @@ class IMAPSSocksRelay(SSLServerMixin, IMAPSocksRelay):
                 # Shut down TLS connection
                 self.socksSocket.shutdown()
                 return False
-        except Exception, e:
+        except Exception as e:
             LOG.debug('IMAPS: %s' % str(e))
             return False
         # Change our outgoing socket to the SSL object of IMAP4_SSL
